#ifndef _AutoResize_h_
#define _AutoResize_h_

#include <CtrlLib/CtrlLib.h>
using namespace Upp;

#include <type_traits>

template<typename T>
class AutoResize : public T
{
	static_assert(std::is_base_of<Ctrl, T>::value, "T must inherit from Ctrl");

private:
	bool noParent = false;
	bool initialization = true;
	Rect initializationRect = Rect(0, 0, 0, 0);
	Rect topCtrlInitializationRect = Rect(0, 0, 0, 0);
	TopWindow* topCtrl;

public:
	AutoResize<T>() : T() {}
	
	virtual void Layout()
	{
		if(noParent)
			return;
		
		Rect currentRect = GetRect();
		if(initialization && currentRect != Rect(0, 0, 0, 0))
		{
			if(GetParent() == NULL)
			{
				noParent = true;
				return;
			}
			topCtrl = GetTopWindow();
			topCtrlInitializationRect = topCtrl->GetRect();
			initializationRect = currentRect;
			initialization = false;
		}
		
		float sizeXChange = (float)topCtrl->GetRect().Width()	/ (float)topCtrlInitializationRect.Width();
		float sizeYChange = (float)topCtrl->GetRect().Height()	/ (float)topCtrlInitializationRect.Height();
		
		float rectLeft	= initializationRect.left * sizeXChange;
		float rectTop	= initializationRect.top * sizeYChange;
		float rectSizeX	= initializationRect.Width() * sizeXChange;
		float rectSizeY	= initializationRect.Height() * sizeYChange;
		
		SetRect(rectLeft, rectTop,
				rectSizeX, rectSizeY);
	}
};

#endif
