#ifndef _AutoResizeTopWindow_h_
#define _AutoResizeTopWindow_h_

#include <CtrlLib/CtrlLib.h>
using namespace Upp;

class AutoResizeTopWindow : public TopWindow
{
private:
	bool					m_initialized			= false;
	float					m_initWidth				= 0.0f;
	float					m_initHeight			= 0.0f;
	ArrayMap<Ctrl*, Rect>	m_initRects;
	
	float					m_sizeXChange			= 0.0f;
	float					m_sizeYChange			= 0.0f;
	
	static const int		m_defaultMenuBarHeight	= 20;
	static const int		m_defaultToolBarHeight	= 25;
	int						m_menuBarCount			= 0;
	int						m_toolBarCount			= 0;
	
	enum struct BarType : int
	{
		NO_BAR		= 0,
		ANY_BAR		= 1,
		MENU_BAR	= 2,
		TOOL_BAR	= 3,
	};

public:
	AutoResizeTopWindow() {}
	
	void InitResizer()
	{
		m_initialized = true;
		m_initWidth = GetRect().Width();
		m_initHeight = GetRect().Height();
		m_initRects.Clear();
	}
	
	void Layout()
	{
		if(!m_initialized)
			return;
		
		m_sizeXChange = (float)GetRect().Width() / m_initWidth;
		// TODO: Make it use the actual height of the bar
		m_sizeYChange = ((float)GetRect().Height() - (m_menuBarCount * m_defaultMenuBarHeight) - (m_toolBarCount * m_defaultToolBarHeight)) / (m_initHeight - (m_menuBarCount * m_defaultMenuBarHeight) - (m_toolBarCount * m_defaultToolBarHeight));
		
		ResizeChildrenCtrls(this);
	}
	
	virtual void ChildAdded(Ctrl* _child)
	{
		BarType isBar = IsBarCtrl(_child);
		switch(isBar)
		{
		case BarType::ANY_BAR:
			break;
		case BarType::MENU_BAR:
			m_menuBarCount++;	// TODO: Make it use the actual height of the bar
			break;
		case BarType::TOOL_BAR:
			m_toolBarCount++;	// TODO: Make it use the actual height of the bar
			break;
		default: // BarType::NO_BAR
			m_initRects.Add(_child, _child->GetRect());
			break;
		}
	}
	
	virtual void ChildRemoved(Ctrl* _child)
	{
		BarType isBar = IsBarCtrl(_child);
		switch(isBar)
		{
		case BarType::ANY_BAR:
			break;
		case BarType::MENU_BAR:
			m_menuBarCount--;	// TODO: Make it use the actual height of the bar
			break;
		case BarType::TOOL_BAR:
			m_toolBarCount--;	// TODO: Make it use the actual height of the bar
			break;
		default: // BarType::NO_BAR
			m_initRects.RemoveKey(_child);
			break;
		}
	}

private:
	void ResizeChildrenCtrls(Ctrl* _parent)
	{
		int childCount = _parent->GetChildCount();
		for(int i = 0; i < childCount; i++)
		{
			Ctrl* currentChild = _parent->GetIndexChild(i);
			
			/*if(m_initRects.Find(currentChild) < 0)
				ChildAdded(currentChild);*/
			
			if(IsBarCtrl(currentChild) != BarType::NO_BAR)
				continue;
			
			if(m_initRects.Find(currentChild) < 0)
				ChildAdded(currentChild);
			
			float rectLeft	= m_initRects.Get(currentChild).left * m_sizeXChange;
			float rectTop	= m_initRects.Get(currentChild).top * m_sizeYChange;
			float rectSizeX	= m_initRects.Get(currentChild).Width() * m_sizeXChange;
			float rectSizeY	= m_initRects.Get(currentChild).Height() * m_sizeYChange;
			
			currentChild->SetRect(rectLeft, rectTop,
								  rectSizeX, rectSizeY);
			
			ResizeChildrenCtrls(currentChild);
		}
	}
	
	BarType IsBarCtrl(Ctrl* _ctrl)
	{
		Bar* barCtrl = dynamic_cast<Bar*>(_ctrl);
		if(barCtrl)
		{
			MenuBar* menuBarCtrl = dynamic_cast<MenuBar*>(_ctrl);
			if(menuBarCtrl)
			{
				return BarType::MENU_BAR;
			}
			ToolBar* toolBarCtrl = dynamic_cast<ToolBar*>(_ctrl);
			if(toolBarCtrl)
			{
				return BarType::TOOL_BAR;
			}
			return BarType::ANY_BAR;
		}
		return BarType::NO_BAR;
	}
};

#endif